<?php // $Id$

/**
 * @file
 *  Link general private message module functionalities to services module.
 */

/**
 * Get all of a users private messages.
 *
 * @param <int> $uid
 *   A user ID for whom's messages to return.
 *
 * @param <string> $type
 * Which type of messages you would like to retrieve
 * possibilities are inbox or sent.
 *
 * @param <bool> $load_full
 * Boolean. Whether to load the full message for each message or not.
 *
 * @return
 *   An array of messages.
 *
 */
function pm_service_get($uid, $type, $load_full = FALSE) {
  $messages = array();
  $pms = array();

  // if no type passed set default type to inbox.
  if (!$type) {
    $type = 'inbox';
  }
  elseif ($type != 'inbox' && $type != 'sent') {
    return services_error(t('Invalid type passed'), 400);
  } 
  // load the user account.
  $account = user_load($uid);
  if (empty($account)) {
    return services_error(t('There is no user with such ID.'), 404);
  }
  else {
    module_load_include('inc', 'privatemsg', 'privatemsg.pages');
    // construct the query and retrieve the correct set of messages.
    $query = _privatemsg_assemble_query('list', $account, $type);
    $result = db_query($query['query']);
    while ($row = db_fetch_object($result)) {
      $pms[$row->thread_id] = $row;
    }
    // check whether we are loading the full message or not.
    if (!empty($pms) && $load_full) {
      $pmids = array_keys($pms);
      $messages = privatemsg_message_load_multiple($pmids, $account);
    }
    else {
      $messages = $pms;
    }
  }
  // return messages.
  return $messages;
}

/**
 * Get the number of unread private messages for a specific user.
 * @param <int> $uid
 *   A user UID to get the number of unread messages for.
 *
 * @return
 *   The unread count.
 */
function pm_service_unread_count($uid) {
  $account = user_load($uid);
  if (empty($account)) {
    return services_error(t('There is no user with such ID.'), 404);
  }
  else {
    module_load_include('inc', 'privatemsg', 'privatemsg.pages');
    $query = _privatemsg_assemble_query('unread_count', $account);
    $unread_count = db_result(db_query($query['query']));
  }
  // return unread count.
  return $unread_count;
}

/**
 * Send a private message to one or more recipients.
 * @param <int> $author
 * The User ID for the author of the message.
 *
 * @param <string> $recipients
 * A comma separated list of usernames for recipients of the message.
 *
 * @param <string> $subject
 * A message subject
 *
 * @param <string> $body
 * A message body
 *
 * @param <int> $thread_id
 * A thread ID. pass this parameter if you are sending a message reply.
 *
 * @return <bool>
 * Return TRUE if sending the message was successful.
 */
function pm_service_send($author, $recipients, $subject, $body = '', $thread_id = NULL) {
  module_load_include('inc', 'privatemsg', 'privatemsg.pages');
  $form_state = array();
  // validate the message author actually exists + is not anonymous user.
  $account = user_load($author);
  if (empty($account) || $account->uid == 0) {
    return services_error(t('There is no user with such ID.'), 404);
  }
  // validate at least 1 recipient has been passed in.
  if ($recipients == '' && !$thread_id) {
    return services_error(t('There are no recipients, please enter a recipient for the message.'), 400);
  }
  else {
    // build up the message params.
    $form_state['values']['author'] = $account;
    $form_state['values']['subject'] = $subject;
    $form_state['values']['body'] = $body;
    $form_state['values']['recipient'] = $recipients;
    if ($thread_id) {
      $form_state['values']['thread_id'] = $thread_id;
    }
    // validate the message and get any errors.
    privatemsg_new_validate(null, $form_state);
    $validation_errors = drupal_get_messages('error');
    if (!empty($validation_errors['error'])) {
      // return - message validation errors.
      return services_error(implode("\n", $validation_errors['error']), 400);
    }
    else {
      // message validated, send the message.
      $status = _privatemsg_send($form_state['validate_built_message']);
      if ($status !== FALSE )  {
        _privatemsg_handle_recipients($status['mid'], $status['recipients']);
      }
      else {
        return services_error(t('An attempt to send a message <em>may have failed</em>', 500));
      }
    }
    // if we get here the message sending was a success.
    return TRUE;
  }
}


